@@ -1,4 +1,4 @@
-/* $Id: ai.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: ai.cpp 29170 2008-09-01 17:41:59Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -266,9 +266,10 @@ bool ai::recruit_usage(const std::string& usage)
 		const std::string& name = i->second.id();
 		// If usage is empty consider any unit.
 		if (i->second.usage() == usage || usage == "") {
+			if (recruits.count(name) == 0)
+				continue;
 			found = true;
-			if(recruits.count(name)
-				&& current_team().gold() - i->second.cost() > min_gold
+			if(current_team().gold() - i->second.cost() > min_gold
 				&& not_recommended_units_.count(name) == 0)
 			{
 				LOG_AI << "recommending '" << name << "'\n";
@@ -287,6 +288,12 @@ bool ai::recruit_usage(const std::string& usage)
 	} else {
 		WRN_AI << "Trying to recruit a: " << usage
 			<< " but no unit of that type (usage=) is available.\n";
+
+		if (usage != "")
+		{
+			return current_team().remove_recruitment_pattern_entry(usage);
+			// remove this recruitment pattern and try again
+		}
 	}
 	return false;
 }
@@ -461,11 +468,11 @@ gamemap::location ai_interface::move_unit_partial(location from, location to,
 			}
 
 			if(steps.size()>1) { // First step is starting hex
-			  unit_map::const_iterator utest=info_.units.find(*(steps.begin()+1));
-			  if(utest != info_.units.end() && current_team().is_enemy(utest->second.side())){
-			    LOG_STREAM(err, ai) << "AI tried to move onto existing enemy unit at"<<*(steps.begin())<<"\n";
-			    //			    return(from);
-			  }
+				unit_map::const_iterator utest=info_.units.find(*(steps.begin()+1));
+				if(utest != info_.units.end() && current_team().is_enemy(utest->second.side())){
+					LOG_STREAM(err, ai) << "AI tried to move onto existing enemy unit at"<<*(steps.begin())<<"\n";
+					//			    return(from);
+				}
 
 				// Check if there are any invisible units that we uncover
 				for(std::vector<location>::iterator i = steps.begin()+1; i != steps.end(); ++i) {
@@ -481,27 +488,29 @@ gamemap::location ai_interface::move_unit_partial(location from, location to,
 
 						// Or would it?  If it doesn't cheat, it might...
 						const unit_map::const_iterator u = info_.units.find(adj[n]);
-						if (u != info_.units.end() && u->second.emits_zoc()
-							&& current_team().is_enemy(u->second.side())) {
+						// If level 0 is invisible it ambush us too
+						if (u != info_.units.end() && (u->second.emits_zoc()||u->second.invisible(adj[n], info_.units, info_.teams))
+								&& current_team().is_enemy(u->second.side())) {
 							if (u->second.invisible(adj[n], info_.units, info_.teams)) {
 								to = *i;
+								u->second.ambush(adj[n]);
 								steps.erase(i,steps.end());
 								break;
 							} else {
-							  if (!u_it->second.get_ability_bool("skirmisher",*i)){
-							    LOG_STREAM(err, ai) << "AI tried to skirmish with non-skirmisher\n";
-							    LOG_AI << "\tresetting destination from " <<to;
-							    to = *i;
-							    LOG_AI << " to " << to;
-							    steps.erase(i,steps.end());
-							    while(steps.empty() == false && (!(info_.units.find(to) == info_.units.end() || from == to))){
-								 to = *(steps.end()-1);
-								 steps.pop_back();
-								 LOG_AI << "\tresetting to " << from << " -> " << to << '\n';
-							    }
+								if (!u_it->second.get_ability_bool("skirmisher",*i)){
+									LOG_STREAM(err, ai) << "AI tried to skirmish with non-skirmisher\n";
+									LOG_AI << "\tresetting destination from " <<to;
+									to = *i;
+									LOG_AI << " to " << to;
+									steps.erase(i,steps.end());
+									while(steps.empty() == false && (!(info_.units.find(to) == info_.units.end() || from == to))){
+										to = *(steps.end()-1);
+										steps.pop_back();
+										LOG_AI << "\tresetting to " << from << " -> " << to << '\n';
+									}
 
-							    break;
-							  }
+									break;
+								}
 							}
 						}
 					}
@@ -1759,12 +1768,12 @@ void ai::do_recruitment()
 
 	// If there is no recruitment_pattern use "" which makes us consider
 	// any unit available.
-	if (options.empty()) {
-		options.push_back("");
-	}
-	// Buy units as long as we have room and can afford it.
-	while(recruit_usage(options[rand()%options.size()])) {
-	}
+	do {
+		if (options.empty()) {
+			options.push_back("");
+		}
+		// Buy units as long as we have room and can afford it.
+	}while(recruit_usage(options[rand()%options.size()]));
 }
 
 void ai::move_leader_to_goals( const move_map& enemy_dstsrc)
