@@ -1,4 +1,4 @@
-/* $Id: campaign_server.cpp 27440 2008-06-23 16:31:36Z suokko $ */
+/* $Id: campaign_server.cpp 28352 2008-08-06 16:46:45Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -318,7 +318,7 @@ namespace {
 				config data;
 				while((sock = network::receive_data(data, 0, &gzipped)) != network::null_connection) {
 					if(const config* req = data.child("request_campaign_list")) {
-						LOG_CS << "sending campaign list to " << network::ip_address(sock) << (gzipped?" using gzip":"") << "\n";
+						LOG_CS << "sending campaign list to " << network::ip_address(sock) << (gzipped?" using gzip":"");
 						time_t epoch = time(NULL);
 						config campaign_list;
 						(campaign_list)["timestamp"] = lexical_cast<std::string>(epoch);
@@ -373,28 +373,26 @@ namespace {
 						config response;
 						response.add_child("campaigns",campaign_list);
 
-						network::send_data(response, sock, gzipped);
+						std::cerr << " size: " << (network::send_data(response, sock, gzipped)/1024) << "kb\n";
 					} else if(const config* req = data.child("request_campaign")) {
-						LOG_CS << "sending campaign '" << (*req)["name"] << "' to " << network::ip_address(sock) << (gzipped?" using gzip":"") << "\n";
+						LOG_CS << "sending campaign '" << (*req)["name"] << "' to " << network::ip_address(sock) << (gzipped?" using gzip":"");
 						config* const campaign = campaigns().find_child("campaign","name",(*req)["name"]);
 						if(campaign == NULL) {
 
 							network::send_data(construct_error("Add-on '" + (*req)["name"] + "'not found."), sock, gzipped);
+							std::cerr << "\n";
 						} else {
 							if (gzipped)
 							{
+								std::cerr << " size: " << (file_size((*campaign)["filename"])/1024) << "kb\n";
 								network::send_file((*campaign)["filename"], sock);
-#if 0
-								util::scoped_resource<char*,util::delete_array> buf(new char[size+1]);
-								stream->read(buf,size);
-
-								network::send_raw_data(buf, size, sock);
-#endif
 							} else {
 								scoped_istream stream = istream_file((*campaign)["filename"]);
 								config cfg;
 								read_gz(cfg, *stream);
-								network::send_data(cfg, sock, false);
+								std::cerr << " size: " <<
+									(network::send_data(cfg, sock, false)/1024)
+									<< "kb\n";
 							}
 
 							const int downloads = lexical_cast_default<int>((*campaign)["downloads"],0)+1;
