@@ -1,4 +1,4 @@
-/* $Id: config.hpp 23936 2008-02-18 19:17:49Z suokko $ */
+/* $Id: config.hpp 24568 2008-03-12 15:14:54Z soliton $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -145,6 +145,7 @@ public:
 	//! I.e. calling cfg2.apply_diff(cfg1.get_diff(cfg2))
 	//! will make cfg1 identical to cfg2.
 	config get_diff(const config& c) const;
+	void get_diff(const config& c, config& res) const;
 
 	void apply_diff(const config& diff); //throw error
 
@@ -168,6 +169,9 @@ public:
 	//! Resets the translated values of all strings contained in this object
 	void reset_translation() const;
 
+	//this is a cheap O(1) operation
+	void swap(config& cfg);
+
 	//! All the attributes of this node.
 	string_map values;
 
