@@ -1,4 +1,4 @@
-/* $Id: game_preferences.cpp 24653 2008-03-14 22:26:46Z noyga $ */
+/* $Id: game_preferences.cpp 28552 2008-08-13 14:57:04Z ivanovic $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -259,7 +259,7 @@ const std::string campaign_server()
 	if(!preferences::get("campaign_server").empty()) {
 		return preferences::get("campaign_server");
 	} else {
-		return "campaigns.wesnoth.org";
+		return "add-ons.wesnoth.org";
 	}
 }
 
