@@ -1,10 +1,11 @@
-/* $Id: key.cpp,v 1.16 2005/06/04 19:16:05 ott Exp $ */
+/* $Id: key.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,13 +15,15 @@
 #include "global.hpp"
 #include "key.hpp"
 
-CKey::CKey() : is_enabled(true)
+CKey::CKey() : 
+	key_list(0),
+	is_enabled(true)
 {
 	static int num_keys = 300;
 	key_list = SDL_GetKeyState( &num_keys );
 }
 
-int CKey::operator[]( int code )
+int CKey::operator[]( int code ) const
 {
 	return (code == SDLK_ESCAPE || is_enabled) && int(key_list[code]);
 }
