@@ -1,10 +1,11 @@
-/* $Id: key.hpp 7396 2005-07-02 21:37:20Z ott $ */
+/* $Id: key.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -25,7 +26,7 @@ class CKey {
 public:
 	CKey();
 
-	int operator[](int);
+	int operator[](int) const;
 	void SetEnabled(bool enable);
 private:
 	Uint8 *key_list;
