@@ -1,4 +1,4 @@
-/* $Id: playmp_controller.cpp 27532 2008-06-27 16:44:25Z suokko $ */
+/* $Id: playmp_controller.cpp 29120 2008-08-30 16:04:46Z suokko $ */
 /*
    Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
@@ -23,6 +23,8 @@
 #include "sound.hpp"
 #include "upload_log.hpp"
 
+#include "SDL.h"
+
 #include <cassert>
 
 #define LOG_NG LOG_STREAM(info, engine)
@@ -182,6 +184,9 @@ void playmp_controller::play_human_turn(){
 				try{
 					if (turn_data_->process_network_data(cfg,res,backlog,skip_replay_) == turn_info::PROCESS_RESTART_TURN)
 					{
+						// Clean undo stack if turn has to be restarted (losing control)
+						while(!undo_stack_.empty())
+							menu_handler_.undo(gui_->get_playing_team() + 1);
 						throw end_turn_exception(gui_->get_playing_team() + 1);
 					}
 				}
