@@ -1,10 +1,11 @@
-/* $Id: publish_campaign.hpp 14156 2006-11-05 11:09:59Z elias $ */
+/* $Id: publish_campaign.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -23,6 +24,7 @@ void get_campaign_info(const std::string& campaign_name, class config& cfg);
 void set_campaign_info(const std::string& campaign_name, const class config& cfg);
 
 std::vector<std::string> available_campaigns();
+std::vector<std::string> installed_campaigns();
 void archive_campaign(const std::string& campaign_name, class config& cfg);
 void unarchive_campaign(const class config& cfg);
 
