@@ -1,10 +1,12 @@
+/* $Id: replay_controller.cpp 24208 2008-03-02 11:40:03Z mordante $ */
 /*
-   Copyright (C) 2006 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playlevel Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -17,6 +19,7 @@
 #include "config_adapter.hpp"
 #include "cursor.hpp"
 #include "filesystem.hpp"
+#include "game_preferences.hpp"
 #include "gettext.hpp"
 #include "game_events.hpp"
 #include "halo.hpp"
@@ -24,17 +27,20 @@
 #include "intro.hpp"
 #include "log.hpp"
 #include "map_create.hpp"
-#include "preferences.hpp"
 #include "preferences_display.hpp"
 #include "replay.hpp"
 #include "replay_controller.hpp"
 #include "sound.hpp"
 #include "tooltips.hpp"
+#include "video.hpp"
 
 #include <iostream>
 #include <iterator>
 
-#define LOG_NG LOG_STREAM(info, engine)
+#define DBG_NG LOG_STREAM(debug, engine)
+#define DBG_REPLAY LOG_STREAM(debug, replay)
+#define LOG_REPLAY LOG_STREAM(info, replay)
+#define ERR_REPLAY LOG_STREAM(err, replay)
 
 LEVEL_RESULT play_replay_level(const game_data& gameinfo, const config& game_config,
 		const config* level, CVideo& video, game_state& state_of_game)
@@ -42,9 +48,9 @@ LEVEL_RESULT play_replay_level(const game_data& gameinfo, const config& game_con
 	try{
 		const int ticks = SDL_GetTicks();
 		const int num_turns = atoi((*level)["turns"].c_str());
-		LOG_NG << "creating objects... " << (SDL_GetTicks() - ticks) << "\n";
+		DBG_NG << "creating objects... " << (SDL_GetTicks() - ticks) << "\n";
 		replay_controller replaycontroller(*level, gameinfo, state_of_game, ticks, num_turns, game_config, video);
-		LOG_NG << "created objects... " << (SDL_GetTicks() - replaycontroller.get_ticks()) << "\n";
+		DBG_NG << "created objects... " << (SDL_GetTicks() - replaycontroller.get_ticks()) << "\n";
 		const events::command_disabler disable_commands;
 
 		//replay event-loop
@@ -53,7 +59,8 @@ LEVEL_RESULT play_replay_level(const game_data& gameinfo, const config& game_con
 		}
 	}
 	catch(end_level_exception&){
-		LOG_NG << "play_replay_level: end_level_exception\n";
+		DBG_NG << "play_replay_level: end_level_exception\n";
+	} catch (replay::error&) {
 	}
 
 	return LEVEL_CONTINUE;
@@ -62,94 +69,96 @@ LEVEL_RESULT play_replay_level(const game_data& gameinfo, const config& game_con
 replay_controller::replay_controller(const config& level, const game_data& gameinfo, game_state& state_of_game,
 						   const int ticks, const int num_turns, const config& game_config,
 						   CVideo& video)
-	: play_controller(level, gameinfo, state_of_game, ticks, num_turns, game_config, video, false),
+	: play_controller(level, gameinfo, state_of_game, ticks, num_turns, game_config, video, false, true),
 	  gamestate_start_(state_of_game), status_start_(level, num_turns, &state_of_game)
 {
 	current_turn_ = 1;
 	delay_ = 0;
 	is_playing_ = false;
+	show_everything_ = false;
+	show_team_ = 1;
 	init();
 	gamestate_start_ = gamestate_;
 }
 
 replay_controller::~replay_controller(){
+	//YogiHH
+	//not absolutely sure if this is needed, but it makes me feel a lot better ;-)
+	//feel free to delete this if it is not necessary
+	gui_->get_theme().theme_reset().detach_handler(this);
+}
+
+bool replay_controller::continue_replay() {
+	return !gui::dialog(*gui_,"",_("The file you have tried to load is corrupt."
+			" Continue playing?"),gui::OK_CANCEL).show();
 }
 
 void replay_controller::init(){
-	LOG_NG << "in replay_controller::init()...\n";
+	DBG_REPLAY << "in replay_controller::init()...\n";
 
 	//guarantee the cursor goes back to 'normal' at the end of the level
 	const cursor::setter cursor_setter(cursor::NORMAL);
 	init_replay_display();
 
-	fire_prestart(true);
+	try {
+		fire_prestart(true);
+	} catch (replay::error&) {
+		if(!continue_replay()) {
+			throw;
+		}
+	}
 	init_gui();
 	statistics::fresh_stats();
+	victory_conditions::set_victory_when_enemies_defeated(
+						level_["victory_when_enemies_defeated"] != "no");
 
-	LOG_NG << "first_time..." << (recorder.is_skipping() ? "skipping" : "no skip") << "\n";
+	DBG_REPLAY << "first_time..." << (recorder.is_skipping() ? "skipping" : "no skip") << "\n";
 
-	fire_start(!loading_game_);
+	try {
+		fire_start(!loading_game_);
+	} catch (replay::error&) {
+		if(!continue_replay()) {
+			throw;
+		}
+	}
 	update_gui();
 
 	units_start_ = units_;
-	teams_start_ = team_manager_.clone(teams_);
+	teams_start_ = teams_;
 }
 
 void replay_controller::init_gui(){
-	LOG_NG << "Initializing GUI... " << (SDL_GetTicks() - ticks_) << "\n";
+	DBG_NG << "Initializing GUI... " << (SDL_GetTicks() - ticks_) << "\n";
 	play_controller::init_gui();
 
-	gui_->scroll_to_leader(units_, player_number_);
+	if (show_team_)
+		gui_->set_team(show_team_ - 1, show_everything_);
+
+	gui_->scroll_to_leader(units_, player_number_, display::WARP);
 	update_locker lock_display((*gui_).video(),false);
-	init_shroudfog_controls(teams_.begin());
 	for(std::vector<team>::iterator t = teams_.begin(); t != teams_.end(); ++t) {
 		t->reset_objectives_changed();
 	}
 }
 
-void replay_controller::init_shroudfog_controls(const std::vector<team>::iterator t){
-	gui::button* b = gui_->find_button("check-fog");
-	if (b != NULL) { b->set_check(t->uses_fog()); }
-	b = gui_->find_button("check-shroud");
-	if (b != NULL) { b->set_check(t->uses_shroud()); }
-}
-
 void replay_controller::init_replay_display(){
-	LOG_NG << "initializing replay-display... " << (SDL_GetTicks() - ticks_) << "\n";
+	DBG_REPLAY << "initializing replay-display... " << (SDL_GetTicks() - ticks_) << "\n";
+
+	rebuild_replay_theme();
+	gui_->get_theme().theme_reset().attach_handler(this);
+	DBG_REPLAY << "done initializing replay-display... " << (SDL_GetTicks() - ticks_) << "\n";
+}
+
+void replay_controller::rebuild_replay_theme(){
 	const config* theme_cfg = get_theme(game_config_, level_["theme"]);
-	const config* replay_theme_cfg = theme_cfg->child("resolution")->child("replay");
-	if (NULL != replay_theme_cfg)
-	    gui_->get_theme().modify(replay_theme_cfg);
-	gui_->invalidate_theme();
-	LOG_NG << "done initializing replay-display... " << (SDL_GetTicks() - ticks_) << "\n";
-}
-
-std::vector<team>& replay_controller::get_teams(){
-	return teams_;
-}
-
-unit_map replay_controller::get_units(){
-	return units_;
-}
-
-display& replay_controller::get_gui(){
-	return *gui_;
-}
-
-gamemap& replay_controller::get_map(){
-	return map_;
-}
-
-const gamestatus& replay_controller::get_status(){
-	return status_;
-}
-
-const int replay_controller::get_player_number(){
-	return player_number_;
-}
-
-const bool replay_controller::is_loading_game(){
-	return loading_game_;
+	if (theme_cfg) {
+		const config* replay_theme_cfg = theme_cfg->child("resolution")->child("replay");
+		if (NULL != replay_theme_cfg)
+			gui_->get_theme().modify(replay_theme_cfg);
+		//Make sure we get notified if the theme is redrawn completely. That way we have
+		//a chance to restore the replay controls of the theme as well.
+		gui_->invalidate_theme();
+	}
 }
 
 void replay_controller::reset_replay(){
@@ -157,6 +166,7 @@ void replay_controller::reset_replay(){
 	if (b != NULL) { b->release(); }
 	b = gui_->find_button("button-stopreplay");
 	if (b != NULL) { b->release(); }
+	gui_->clear_chat_messages();
 	is_playing_ = false;
 	player_number_ = 1;
 	current_turn_ = 1;
@@ -164,16 +174,31 @@ void replay_controller::reset_replay(){
 	units_ = units_start_;
 	status_ = status_start_;
 	gamestate_ = gamestate_start_;
-	teams_ = team_manager_.clone(teams_start_);
+	teams_ = teams_start_;
 	statistics::fresh_stats();
 	if (events_manager_ != NULL){
 		delete events_manager_;
-		events_manager_ = new game_events::manager(level_,*gui_,map_,units_,teams_,
-											gamestate_,status_,gameinfo_);
+		events_manager_ = new game_events::manager(level_,*gui_,map_, *soundsources_manager_,
+								units_,teams_, gamestate_,status_,gameinfo_);
 	}
-	init_shroudfog_controls(teams_.begin());
-	fire_prestart(true);
-	fire_start(!loading_game_);
+
+	gui_->new_turn();
+	gui_->invalidate_game_status();
+	events::raise_draw_event();
+	(*gui_).invalidate_all();
+	(*gui_).draw();
+
+	try {
+		fire_prestart(true);
+		fire_start(!loading_game_);
+	} catch (replay::error&) {
+		if(!continue_replay()) {
+			throw;
+		}
+	}
+	gui_->new_turn();
+	gui_->invalidate_game_status();
+	events::raise_draw_event();
 	(*gui_).invalidate_all();
 	(*gui_).draw();
 	b = gui_->find_button("button-resetreplay");
@@ -189,6 +214,10 @@ void replay_controller::stop_replay(){
 void replay_controller::replay_next_turn(){
 	is_playing_ = true;
 	play_turn();
+
+ 	if (!skip_replay_){
+		gui_->scroll_to_leader(units_, player_number_);
+	}
 	is_playing_ = false;
 	gui::button* b = gui_->find_button("button-nextturn");
 	if (b != NULL) { b->release(); }
@@ -198,29 +227,38 @@ void replay_controller::replay_next_side(){
 	is_playing_ = true;
 	play_side(player_number_ - 1, false);
 
-	if ((size_t)player_number_ > teams_.size()) {
+	if (static_cast<size_t>(player_number_) > teams_.size()) {
 		player_number_ = 1;
 		current_turn_++;
 	}
+
+	if (!skip_replay_) {
+		gui_->scroll_to_leader(units_, player_number_);
+	}
+
 	is_playing_ = false;
 	gui::button* b = gui_->find_button("button-nextside");
 	if (b != NULL) { b->release(); }
 }
 
-void replay_controller::replay_switch_fog(){
-	gui::button* b = gui_->find_button("check-fog");
-	for(std::vector<team>::iterator t = teams_.begin(); t != teams_.end(); ++t) {
-		t->set_fog(b->checked());
-	}
+void replay_controller::replay_show_everything(){
+	show_everything_ = true;
+	show_team_ = 0;
 	update_teams();
 	update_gui();
 }
 
-void replay_controller::replay_switch_shroud(){
-	gui::button* b = gui_->find_button("check-shroud");
-	for(std::vector<team>::iterator t = teams_.begin(); t != teams_.end(); ++t) {
-		t->set_shroud(b->checked());
-	}
+void replay_controller::replay_show_each(){
+	show_everything_ = false;
+	show_team_ = 0;
+	update_teams();
+	update_gui();
+}
+
+void replay_controller::replay_show_team1(){
+	show_everything_ = false;
+	show_team_ = 1;
+	gui_->set_team(show_team_ - 1, show_everything_);
 	update_teams();
 	update_gui();
 }
@@ -240,12 +278,10 @@ void replay_controller::play_replay(){
 	try{
 		is_playing_ = true;
 
-		LOG_NG << "starting main loop\n" << (SDL_GetTicks() - ticks_) << "\n";
-		for(; !recorder.at_end() && is_playing_; first_player_ = 1
-			) {
+		DBG_REPLAY << "starting main loop\n" << (SDL_GetTicks() - ticks_) << "\n";
+		for(; !recorder.at_end() && is_playing_; first_player_ = 1) {
 			try{
-			play_turn();
-			play_slice();
+				play_turn();
 			}
 			catch (replay::error&) //when user due to error want stop playing
 			{
@@ -264,71 +300,91 @@ void replay_controller::play_turn(){
 		return;
 	}
 
-	LOG_NG << "turn: " << current_turn_ << "\n";
+	LOG_REPLAY << "turn: " << current_turn_ << "\n";
 
-	while ( ((size_t)player_number_ <= teams_.size()) && (!recorder.at_end()) ){
+	gui_->new_turn();
+	gui_->invalidate_game_status();
+	events::raise_draw_event();
+
+	bool last_team = false;
+
+	while ( (!last_team) && (!recorder.at_end()) && is_playing_ ){
+		last_team = static_cast<size_t>(player_number_) == teams_.size();
 		play_side(player_number_ - 1, false);
+		play_slice();
 	}
-
-	player_number_ = 1;
-	current_turn_++;
 }
 
-void replay_controller::play_side(const unsigned int team_index, bool){
+void replay_controller::play_side(const unsigned int /*team_index*/, bool){
 	if (recorder.at_end()){
 		return;
 	}
 
+	DBG_REPLAY << "Status turn number: " << status_.turn() << "\n";
+	DBG_REPLAY << "Replay_Controller turn number: " << current_turn_ << "\n";
+	DBG_REPLAY << "Player number: " << player_number_ << "\n";
+
+	// If a side is empty skip over it.
+	if (current_team().is_empty()) return;
+
+	statistics::reset_turn_stats(player_number_);
+
 	try{
-		play_controller::init_side(team_index, true);
+		play_controller::init_side(player_number_ - 1, true);
 
-		//if a side is dead, don't do their turn
-		if(!current_team().is_empty() && team_units(units_,player_number_) > 0) {
-
-			do_replay(true);
-
-			finish_side_turn();
-
-			for(unit_map::iterator uit = units_.begin(); uit != units_.end(); ++uit) {
-				if(uit->second.side() != (size_t)player_number_){
-					//this is necessary for replays in order to show possible movements
-					uit->second.new_turn();
-				}
+		DBG_REPLAY << "doing replay " << player_number_ << "\n";
+		try {
+			::do_replay(*gui_, map_, gameinfo_, units_, teams_,
+					player_number_, status_, gamestate_);
+		} catch(replay::error&) {
+			if(!continue_replay()) {
+				throw;
 			}
 		}
+
+		finish_side_turn();
+
+		// This is necessary for replays in order to show possible movements.
+		for (unit_map::iterator uit = units_.begin(); uit != units_.end(); ++uit) {
+			if (uit->second.side() != static_cast<size_t>(player_number_)) {
+				uit->second.new_turn();
+			}
+		}
+
+		player_number_++;
+
+		if (static_cast<size_t>(player_number_) > teams_.size()){
+			status_.next_turn();
+			finish_turn();
+			player_number_ = 1;
+			current_turn_++;
+		}
+
+		update_teams();
+		update_gui();
 	}
 	catch (replay::error&) //if replay throws an error, we don't want to get thrown out completely
 	{
 		is_playing_ = false;
 	}
-	catch(end_level_exception& e){ 
+	catch(end_level_exception& e){
 		//VICTORY/DEFEAT end_level_exception shall not return to title screen
 		if ((e.result != VICTORY) && (e.result != DEFEAT)) { throw e; }
 	}
-
-	player_number_++;
-
-	if ((size_t)player_number_ > teams_.size()) {
-		status_.next_turn();
-		finish_turn();
-	}
-	update_teams();
-	update_gui();
 }
 
 void replay_controller::update_teams(){
 	int next_team = player_number_;
-	if ((size_t)next_team > teams_.size()) { next_team = 1; }
-	if (teams_[next_team - 1].uses_fog()){
-		gui_->set_team(next_team - 1);
-		::clear_shroud(*gui_, status_, map_, gameinfo_, units_, teams_, next_team - 1);
-	}
-	if (teams_[next_team - 1].uses_shroud()){
-		gui_->set_team(next_team - 1);
-		recalculate_fog(map_, status_, gameinfo_, units_, teams_, next_team - 1);
+	if(static_cast<size_t>(next_team) > teams_.size()) {
+		next_team = 1;
 	}
+
+	if (!show_team_)
+		gui_->set_team(next_team - 1, show_everything_);
+
+	::clear_shroud(*gui_, map_, units_, teams_,
+		next_team - 1);
 	gui_->set_playing_team(next_team - 1);
-	//(*gui_).scroll_to_leader(units_, next_team);
 	gui_->invalidate_all();
 }
 
@@ -350,9 +406,13 @@ void replay_controller::show_statistics(){
 	menu_handler_.show_statistics(gui_->playing_team()+1);
 }
 
-bool replay_controller::can_execute_command(hotkey::HOTKEY_COMMAND command) const
+void replay_controller::handle_generic_event(const std::string& /*name*/){
+	rebuild_replay_theme();
+}
+
+bool replay_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int index) const
 {
-	bool result = play_controller::can_execute_command(command);
+	bool result = play_controller::can_execute_command(command,index);
 
 	switch(command) {
 
@@ -362,9 +422,13 @@ bool replay_controller::can_execute_command(hotkey::HOTKEY_COMMAND command) cons
 	case hotkey::HOTKEY_STOP_REPLAY:
 	case hotkey::HOTKEY_REPLAY_NEXT_TURN:
 	case hotkey::HOTKEY_REPLAY_NEXT_SIDE:
-	case hotkey::HOTKEY_REPLAY_FOG:
-	case hotkey::HOTKEY_REPLAY_SHROUD:
+	case hotkey::HOTKEY_REPLAY_SHOW_EVERYTHING:
+	case hotkey::HOTKEY_REPLAY_SHOW_EACH:
+	case hotkey::HOTKEY_REPLAY_SHOW_TEAM1:
 	case hotkey::HOTKEY_REPLAY_SKIP_ANIMATION:
+	case hotkey::HOTKEY_SAVE_GAME:
+	case hotkey::HOTKEY_SAVE_REPLAY:
+	case hotkey::HOTKEY_CHAT_LOG:
 		return true;
 
 	default:
