@@ -1,30 +1,39 @@
-/* $Id: scoped_resource.hpp,v 1.19 2005/06/04 19:16:06 ott Exp $ */
+/* $Id: scoped_resource.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
+
+//! @file scoped_resource.hpp
+//! scoped_resource: class template, functions, helper policies etc.\ for
+//! resource management.
+
 #ifndef SCOPED_RESOURCE_H_INCLUDED
 #define SCOPED_RESOURCE_H_INCLUDED
 
+#include "global.hpp"
 #include <cstdio> //for FILE
 
 namespace util
 {
 /**
-* A class template, scoped_resource, designed to
-* implement the Resource Acquisition Is Initialization (RAII) approach
-* to resource management. scoped_resource is designed to be used when
-* a resource is initialized at the beginning or middle of a scope,
-* and released at the end of the scope. The template argument
-* ReleasePolicy is a functor which takes an argument of the
-* type of the resource, and releases it.
+* A class template, scoped_resource, designed to implement
+* the Resource Acquisition Is Initialization (RAII) approach
+* to resource management.
+* scoped_resource is designed to be used when a resource
+* is initialized at the beginning or middle of a scope,
+* and released at the end of the scope.
+* The template argument ReleasePolicy is a functor
+* which takes an argument of the type of the resource,
+* and releases it.
 *
 * Usage example, for working with files:
 *
@@ -37,11 +46,11 @@ namespace util
 * } // file is automatically closed here
 * @endcode
 *
-* Note that scoped_resource has an explicit constructor, and prohibits
-* copy-construction, and thus the initialization syntax, rather than
-* the assignment syntax must be used when initializing.
+* Note that scoped_resource has an explicit constructor,
+* and prohibits copy-construction, and thus the initialization syntax.
+* The assignment syntax must be used when initializing.
 *
-* i.e. using scoped_resource<int,close_file> file = open("file.txt",O_RDONLY);
+* I.e. using scoped_resource<int,close_file> file = open("file.txt",O_RDONLY);
 * in the above example is illegal.
 *
 */
@@ -59,41 +68,42 @@ public:
 
   /**
   * Constructor
-	*
-	* @ param res This is the resource to be managed
+  *
+  * @param res	This is the resource to be managed
   */
 	scoped_resource(resource_type res = resource_type())
 			: resource(res) {}
 
   /**
-  * The destructor is the main point in this class. It takes care of proper
-	* deletion of the resource, using the provided release policy.
-	*/
-	~scoped_resource()
+  * The destructor is the main point in this class.
+  * It takes care of proper deletion of the resource,
+  * using the provided release policy.
+  */
+	virtual ~scoped_resource()
 	{
 		release_type()(resource);
 	}
 
   /**
   * This operator makes sure you can access and use the scoped_resource
-	* just like you were using the resource itself.
+  * just like you were using the resource itself.
   *
-  * @ret the underlying resource
-	*/
+  * @return the underlying resource
+  */
 	operator resource_type() const { return resource; }
 
   /**
-  * This function provides explicit access to the resource. Its behaviour
-  * is identical to operator resource_type()
-	*
-	* @ret the underlying resource
+  * This function provides explicit access to the resource.
+  * Its behaviour is identical to operator resource_type()
+  *
+  * @return the underlying resource
   */
 	resource_type get() const { return resource; }
 
-	/**
+  /**
   * This function provides convenient direct access to the -> operator
-  * if the underlying resource is a pointer. Only call this function
-  * if resource_type is a pointer type.
+  * if the underlying resource is a pointer.
+  * Only call this function if resource_type is a pointer type.
   */
 	resource_type operator->() const { return resource; }
 
@@ -137,9 +147,10 @@ struct delete_array {
 * @endcode
 *
 * NOTE: use this class only to manage a single object, *never* an array.
-* Use scoped_array to manage arrays. This distinction is because you
-* may call delete only on objects allocated with new, delete[] only
-* on objects allocated with new[].
+* Use scoped_array to manage arrays.
+* This distinction is because you may call delete only
+* on objects allocated with new,
+* delete[] only on objects allocated with new[].
 */
 template<typename T>
 struct scoped_ptr : public scoped_resource<T*,delete_item>
@@ -148,8 +159,8 @@ struct scoped_ptr : public scoped_resource<T*,delete_item>
 };
 
 /**
-* This class has identical behaviour to @ref scoped_ptr, except it manages
-* heap-allocated arrays instead of heap-allocated single objects
+* This class has identical behaviour to @ref scoped_ptr, except it
+* manages heap-allocated arrays instead of heap-allocated single objects
 *
 * Usage example:
 * @code
@@ -167,14 +178,14 @@ struct scoped_array : public scoped_resource<T*,delete_array>
 };
 
 /**
- * This class specializes the scoped_resource to implement scoped FILEs. Not
- * sure this is the best place to place such an utility, though.
+ * This class specializes the scoped_resource to implement scoped FILEs.
+ * Not sure this is the best place to place such an utility, though.
  */
 struct close_FILE
 {
-	void operator()(FILE* f) const { if(f != NULL) { fclose(f); } }
+	void operator()(std::FILE* f) const { if(f != NULL) { std::fclose(f); } }
 };
-typedef scoped_resource<FILE*,close_FILE> scoped_FILE;
+typedef scoped_resource<std::FILE*,close_FILE> scoped_FILE;
 
 }
 
