@@ -1,4 +1,4 @@
-/* $Id: server.cpp 27540 2008-06-27 21:30:26Z suokko $ */
+/* $Id: server.cpp 29189 2008-09-02 10:23:49Z soliton $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -27,6 +27,7 @@
 #include "../serialization/preprocessor.hpp"
 #include "../serialization/string_utils.hpp"
 
+#include "ban.hpp"
 #include "game.hpp"
 #include "input_stream.hpp"
 #include "metrics.hpp"
@@ -141,7 +142,7 @@ void make_add_diff(const simple_wml::node& src, const char* gamelist,
 		index = children.size() - 1;
 	}
 
-	assert(index < children.size());
+	assert(static_cast<size_t>(index) < children.size());
 	insert.set_attr_int("index", index);
 	children[index]->copy_into(insert.add_child(type));
 }
@@ -227,15 +228,17 @@ private:
 	network::server_manager server_;
 
 	//! std::map<network::connection,player>
-	player_map players_;
-	std::vector<game*> games_;
-	game not_logged_in_;
+	wesnothd::player_map players_;
+	std::vector<wesnothd::game*> games_;
+	wesnothd::game not_logged_in_;
 	//! The lobby is implemented as a game.
-	game lobby_;
+	wesnothd::game lobby_;
 
 	//! server socket/fifo
 	input_stream& input_;
 
+	wesnothd::ban_manager ban_manager_;
+
 	const std::string config_file_;
 	config cfg_;
 	//! Read the server config from file 'config_file_'.
@@ -280,7 +283,7 @@ private:
 	void process_query(const network::connection sock,
 	                   simple_wml::node& query);
 	//! Process commands from admins and users.
-	std::string process_command(const std::string& cmd);
+	std::string process_command(const std::string& cmd, const std::string& issuer_name);
 	//! Handle private messages between players.
 	void process_whisper(const network::connection sock,
 	                     simple_wml::node& whisper) const;
@@ -288,9 +291,9 @@ private:
 	                        simple_wml::document& data);
 	void process_data_game(const network::connection sock,
 	                       simple_wml::document& data);
-	void delete_game(std::vector<game*>::iterator game_it);
+	void delete_game(std::vector<wesnothd::game*>::iterator game_it);
 
-	void update_game_in_lobby(const game* g, network::connection exclude=0);
+	void update_game_in_lobby(const wesnothd::game* g, network::connection exclude=0);
 
 	void start_new_server();
 };
@@ -312,6 +315,7 @@ server::server(int port, input_stream& input, const std::string& config_file, si
 	last_stats_(last_ping_)
 {
 	load_config();
+	ban_manager_.read();
 	signal(SIGHUP, reload_config);
 	signal(SIGINT, exit_sigint);
 	signal(SIGTERM, exit_sigterm);
@@ -416,12 +420,13 @@ void server::load_config() {
 			proxy_versions_[*j] = **p;
 		}
 	}
+	ban_manager_.load_config(cfg_);
 }
 
 bool server::ip_exceeds_connection_limit(const std::string& ip) const {
 	if (concurrent_connections_ == 0) return false;
 	size_t connections = 0;
-	for (player_map::const_iterator i = players_.begin(); i != players_.end(); ++i) {
+	for (wesnothd::player_map::const_iterator i = players_.begin(); i != players_.end(); ++i) {
 		if (network::ip_address(i->first) == ip) {
 			++connections;
 		}
@@ -431,16 +436,7 @@ bool server::ip_exceeds_connection_limit(const std::string& ip) const {
 }
 
 bool server::is_ip_banned(const std::string& ip) const {
-	for (std::map<std::string,std::string>::const_iterator i = bans_.begin(); i != bans_.end(); ++i) {
-		std::stringstream ss;
-		ss << "Comparing ban '" << i->first << "' vs '..." << ip << "'\t";
-		if (utils::wildcard_string_match(ip, i->first)) {
-			DBG_SERVER << ss.str() << "banned.\n";
-			return true;
-		}
-		DBG_SERVER << ss.str() << "not banned.\n";
-	}
-	return false;
+	return ban_manager_.is_ip_banned(ip);
 }
 
 void server::dump_stats(const time_t& now) {
@@ -465,7 +461,7 @@ void server::run() {
 				// Then client would reconnect to new server automaticaly.
 				// This would also allow server to move to new port or address if there is need
 
-				process_command("msg All games ended. Shutting down now. Reconnect to the new server.");
+				process_command("msg All games ended. Shutting down now. Reconnect to the new server.", "system");
 				throw network::error("shut down");
 			}
 			if (config_reload == 1) {
@@ -476,11 +472,12 @@ void server::run() {
 			// Process commands from the server socket/fifo
 			std::string admin_cmd;
 			if (input_.read_line(admin_cmd)) {
-				process_command(admin_cmd);
+				process_command(admin_cmd, "*socket*");
 			}
 
 			time_t now = time(NULL);
 			if ((loop%100) == 0 && last_ping_ + 10 <= now) {
+				ban_manager_.check_ban_times(now);
 				// Make sure we log stats every 5 minutes
 				if (last_stats_ + 5*60 <= now) dump_stats(now);
 #ifdef BANDWIDTH_MONITOR
@@ -496,7 +493,7 @@ void server::run() {
 				// send a 'ping' to all players to detect ghosts
 				config ping;
 				ping["ping"] = lexical_cast<std::string>(now);
-				for (player_map::const_iterator i = players_.begin();
+				for (wesnothd::player_map::const_iterator i = players_.begin();
 					i != players_.end(); ++i)
 				{
 					network::send_data(ping, i->first, true, "ping");
@@ -590,7 +587,7 @@ void server::run() {
 		} catch(network::error& e) {
 			if (e.message == "shut down") {
 				LOG_SERVER << "Try to disconnect all users...\n";
-				for (player_map::const_iterator pl = players_.begin();
+				for (wesnothd::player_map::const_iterator pl = players_.begin();
 					pl != players_.end(); ++pl)
 				{
 					network::disconnect(pl->first);
@@ -613,7 +610,7 @@ void server::run() {
 				continue;
 			}
 			// Was the user already logged in?
-			const player_map::iterator pl_it = players_.find(e.socket);
+			const wesnothd::player_map::iterator pl_it = players_.find(e.socket);
 			if (pl_it == players_.end()) {
 				if (not_logged_in_.is_observer(e.socket)) {
 					DBG_SERVER << ip << "\tNot logged in user disconnected.\n";
@@ -646,7 +643,7 @@ void server::run() {
 					<< "\thas logged off. (socket: " << e.socket << ")\n";
 				                 
 			} else {
-				for (std::vector<game*>::iterator g = games_.begin();
+				for (std::vector<wesnothd::game*>::iterator g = games_.begin();
 					g != games_.end(); ++g)
 				{
 					if (!(*g)->is_member(e.socket)) {
@@ -801,7 +798,7 @@ void server::process_login(const network::connection sock,
 		}
 	}
 	// Check the username isn't already taken
-	player_map::const_iterator p;
+	wesnothd::player_map::const_iterator p;
 	for (p = players_.begin(); p != players_.end(); ++p) {
 		if (p->second.name() == username) {
 			send_error(sock, "The username you chose is already taken.");
@@ -834,7 +831,7 @@ void server::process_login(const network::connection sock,
 	LOG_SERVER << network::ip_address(sock) << "\t" << username
 		<< "\thas logged on. (socket: " << sock << ")\n";
 
-	for (std::vector<game*>::const_iterator g = games_.begin(); g != games_.end(); ++g) {
+	for (std::vector<wesnothd::game*>::const_iterator g = games_.begin(); g != games_.end(); ++g) {
 		// Note: This string is parsed by the client to identify lobby join messages!
 		(*g)->send_server_message_to_all((username + " has logged into the lobby").c_str());
 	}
@@ -842,7 +839,7 @@ void server::process_login(const network::connection sock,
 
 void server::process_query(const network::connection sock,
                            simple_wml::node& query) {
-	const player_map::const_iterator pl = players_.find(sock);
+	const wesnothd::player_map::const_iterator pl = players_.find(sock);
 	if (pl == players_.end()) {
 		DBG_SERVER << "ERROR: process_query(): Could not find player with socket: " << sock << "\n";
 		return;
@@ -850,20 +847,20 @@ void server::process_query(const network::connection sock,
 	const simple_wml::string_span& command(query["type"]);
 	std::ostringstream response;
 	const std::string& help_msg = "Available commands are: help, metrics,"
-			" motd, status, wml.";
+			" motd, netstats [all], status, wml.";
 	if (admins_.count(sock) != 0) {
 		LOG_SERVER << "Admin Command:" << "\ttype: " << command
 			<< "\tIP: "<< network::ip_address(sock) 
 			<< "\tnick: "<< pl->second.name() << std::endl;
-		response << process_command(command.to_string());
+		response << process_command(command.to_string(), pl->second.name());
 	// Commands a player may issue.
 	} else if (command == "help") {
 		response << help_msg;
 	} else if (command == "status") {
-		response << process_command(command.to_string() + " " + pl->second.name());
+		response << process_command(command.to_string() + " " + pl->second.name(), pl->second.name());
 	} else if (command == "status " + pl->second.name() || command == "metrics"
-	|| command == "motd" || command == "wml" || command == "netstats") {
-		response << process_command(command.to_string());
+	|| command == "motd" || command == "wml" || command == "netstats" || command == "netstats all") {
+		response << process_command(command.to_string(), pl->second.name());
 	} else if (command == admin_passwd_) {
 		LOG_SERVER << "New Admin recognized:" << "\tIP: "
 			<< network::ip_address(sock) << "\tnick: "
@@ -893,17 +890,18 @@ void server::start_new_server() {
 	LOG_SERVER << "New server started with command: " << restart_command << "\n";
 }
 
-std::string server::process_command(const std::string& query) {
+std::string server::process_command(const std::string& query, const std::string& issuer_name) {
 	std::ostringstream out;
 	const std::string::const_iterator i = std::find(query.begin(),query.end(),' ');
 	const std::string command(query.begin(),i);
 	std::string parameters = (i == query.end() ? "" : std::string(i+1,query.end()));
 	utils::strip(parameters);
-	const std::string& help_msg = "Available commands are: ban(s) [<mask>] [<reason>],"
-			"kick <mask>, k(ick)ban [<mask>] [<reason>], help, metrics, netstats,"
+	const std::string& help_msg = "Available commands are: ban(s) [<mask>] [<time>] <reason>,"
+			"kick <mask>, k(ick)ban [<mask>] [<time>] <reason>, help, metrics, netstats,"
 			" (lobby)msg <message>, motd [<message>], status [<mask>],"
-			" unban <ipmask>, shut_down [now], restart";
-	if (command == "shut_down") {
+			" unban <ipmask>";
+	// Shutdown and restart commands can only be issued via the socket.
+	if (command == "shut_down" && issuer_name == "*socket*") {
 		if (parameters == "now") {
 			throw network::error("shut down");
 		} else {
@@ -911,13 +909,13 @@ std::string server::process_command(const std::string& query) {
 			server_.stop();
 			input_.stop();
 			graceful_restart = true;
-			process_command("msg The server is shutting down. You may finish your games but can't start new ones. Once all games have ended the server will exit.");
+			process_command("msg The server is shutting down. You may finish your games but can't start new ones. Once all games have ended the server will exit.", issuer_name);
 			out << "Server is doing graceful shut down.";
 		}
 
 #ifndef _WIN32  // Not sure if this works on windows
 		// TODO: check if this works in windows.
-	} else if (command == "restart") {
+	} else if (command == "restart" && issuer_name == "*socket*") {
 		if (restart_command.empty()) {
 			out << "No restart_command configured! Not restarting.";
 		} else {
@@ -928,7 +926,7 @@ std::string server::process_command(const std::string& query) {
 			input_.stop();
 			// start new server
 			start_new_server();
-			process_command("msg The server has been restarted. You may finish your games but can't start new ones and new players can't join this server.");
+			process_command("msg The server has been restarted. You may finish your games but can't start new ones and new players can't join this server.", issuer_name);
 			out << "New server started.";
 		}
 #endif
@@ -957,7 +955,7 @@ std::string server::process_command(const std::string& query) {
 		}
 		lobby_.send_server_message_to_all(parameters.c_str());
 		if (command == "msg") {
-			for (std::vector<game*>::const_iterator g = games_.begin(); g != games_.end(); ++g) {
+			for (std::vector<wesnothd::game*>::const_iterator g = games_.begin(); g != games_.end(); ++g) {
 				(*g)->send_server_message_to_all(parameters.c_str());
 			}
 		}
@@ -965,7 +963,7 @@ std::string server::process_command(const std::string& query) {
 		out << "message '" << parameters << "' relayed to players\n";
 	} else if (command == "status") {
 		out << "STATUS REPORT\n";
-		for (player_map::const_iterator pl = players_.begin(); pl != players_.end(); ++pl) {
+		for (wesnothd::player_map::const_iterator pl = players_.begin(); pl != players_.end(); ++pl) {
 			if (parameters == ""
 				|| utils::wildcard_string_match(pl->second.name(), parameters)
 				|| utils::wildcard_string_match(network::ip_address(pl->first), parameters)) {
@@ -980,30 +978,56 @@ std::string server::process_command(const std::string& query) {
 					<< stats.bytes_received << " bytes\n";
 			}
 		}
-	} else if (command == "ban" || command == "bans" || command == "kban" || command == "kickban") {
+	} else if (command == "ban" || command == "bans" || command == "kban" || command == "kickban" || command == "gban") {
 		if (parameters == "") {
-			if (bans_.empty()) return "No bans set.";
-			out << "BAN LIST\n";
-			for (std::map<std::string,std::string>::const_iterator i = bans_.begin();
-				i != bans_.end(); ++i)
-			{
-				out << "IP: '" << i->first << "' reason: '" << i->second << "'\n";
-			}
+			ban_manager_.list_bans(out);
 		} else {
-			bool banned = false;
+			bool banned_ = false;
 			const bool kick = (command == "kban" || command == "kickban");
-			const std::string::iterator i = std::find(parameters.begin(), parameters.end(), ' ');
-			const std::string target(parameters.begin(), i);
-			std::string reason = (i == parameters.end() ? "" : std::string(i + 1, parameters.end()));
+			const bool group_ban = command == "gban";
+			std::string::iterator first_space = std::find(parameters.begin(), parameters.end(), ' ');
+			if (first_space == parameters.end())
+			{
+				return ban_manager_.get_ban_help();
+			}
+			std::string::iterator second_space = std::find(first_space+1, parameters.end(), ' ');
+			const std::string target(parameters.begin(), first_space);
+			std::string group;
+			if (group_ban)
+			{
+				group = std::string(first_space+1, second_space);
+				first_space = second_space;
+				second_space = std::find(first_space+1, parameters.end(), ' ');
+			}
+			const std::string time(first_space+1,second_space);
+			time_t parsed_time = ban_manager_.parse_time(time);
+			if (parsed_time == 0)
+			{
+				second_space = first_space;
+			}
+
+			if (second_space == parameters.end())
+			{
+				--second_space;
+			}
+			std::string reason(second_space + 1, parameters.end());
 			utils::strip(reason);
+			if (reason.empty())
+				return ban_manager_.get_ban_help();
+
 			// if we find a '.' consider it an ip mask 
 			//! @todo  FIXME: should also check for only numbers
 			if (std::count(target.begin(), target.end(), '.') >= 1) {
-				banned = true;
-				out << "Set ban on '" << target << "' with reason: '" << reason << "'.\n";
-				bans_[target] = reason;
+				banned_ = true;
+
+				std::string err = ban_manager_.ban(target, parsed_time, reason, issuer_name, group);
+				if (err.empty())
+					out << "Set ban on '" << target << "' with end time '" <<  wesnothd::banned::get_human_end_time(parsed_time) << "'  with reason: '" << reason << "'.\n";
+				else
+					out << err << "\n";
+	
 				if (kick) {
-					for (player_map::const_iterator pl = players_.begin();
+					for (wesnothd::player_map::const_iterator pl = players_.begin();
 						pl != players_.end(); ++pl)
 					{
 						if (utils::wildcard_string_match(network::ip_address(pl->first), target)) {
@@ -1013,16 +1037,19 @@ std::string server::process_command(const std::string& query) {
 					}
 				}
 			} else {
-				for (player_map::const_iterator pl = players_.begin();
+				for (wesnothd::player_map::const_iterator pl = players_.begin();
 					pl != players_.end(); ++pl)
 				{
 					if (utils::wildcard_string_match(pl->second.name(), target)) {
-						banned = true;
+						banned_ = true;
 						const std::string& ip = network::ip_address(pl->first);
 						if (!is_ip_banned(ip)) {
-							bans_[ip] = reason;
-							out << "Set ban on '" << ip << "' with reason: '"
-								<< reason << "'.\n";
+							std::string err = ban_manager_.ban(ip,parsed_time, reason, issuer_name, group);
+							if (err.empty())
+								out << "Set ban on '" << ip << "' with end time '" << wesnothd::banned::get_human_end_time(parsed_time) << "' with reason: '"
+									<< reason << "'.\n";
+							else
+								out << err << "\n";
 						}
 						if (kick) {
 							out << "Kicked " << pl->second.name() << ".\n";
@@ -1030,7 +1057,7 @@ std::string server::process_command(const std::string& query) {
 						}
 					}
 				}
-				if (!banned) {
+				if (!banned_) {
 					out << "Nickmask '" << target << "' did not match, no bans set.";
 				}
 			}
@@ -1039,12 +1066,12 @@ std::string server::process_command(const std::string& query) {
 		if (parameters == "") {
 			return "You must enter an ipmask to unban.";
 		}
-		const int n = bans_.erase(parameters);
-		if (n == 0) {
-			out << "There is no ban on '" << parameters << "'.";
-		} else {
-			out << "Ban on '" << parameters << "' removed.";
+		ban_manager_.unban(out, parameters);
+	} else if (command == "ungban") {
+		if (parameters == "") {
+			return "You must enter an ipmask to unban.";
 		}
+		ban_manager_.unban_group(out, parameters);
 	} else if (command == "kick") {
 		if (parameters == "") {
 			return "You must enter a mask to kick.";
@@ -1052,7 +1079,7 @@ std::string server::process_command(const std::string& query) {
 		bool kicked = false;
 		// if we find a '.' consider it an ip mask
 		if (std::count(parameters.begin(), parameters.end(), '.') >= 1) {
-			for (player_map::const_iterator pl = players_.begin();
+			for (wesnothd::player_map::const_iterator pl = players_.begin();
 				pl != players_.end(); ++pl)
 			{
 				if (utils::wildcard_string_match(network::ip_address(pl->first), parameters)) {
@@ -1062,7 +1089,7 @@ std::string server::process_command(const std::string& query) {
 				}
 			}
 		} else {
-			for (player_map::const_iterator pl = players_.begin();
+			for (wesnothd::player_map::const_iterator pl = players_.begin();
 				pl != players_.end(); ++pl)
 			{
 				if (utils::wildcard_string_match(pl->second.name(), parameters)) {
@@ -1111,7 +1138,7 @@ void server::process_whisper(const network::connection sock,
 		send_doc(data, sock, "error");
 		return;
 	}
-	const player_map::const_iterator pl = players_.find(sock);
+	const wesnothd::player_map::const_iterator pl = players_.find(sock);
 	if (pl == players_.end()) {
 		ERR_SERVER << "ERROR: Could not find whispering player. (socket: "
 			<< sock << ")\n";
@@ -1120,12 +1147,12 @@ void server::process_whisper(const network::connection sock,
 	whisper.set_attr_dup("sender", pl->second.name().c_str());
 	bool dont_send = false;
 	const simple_wml::string_span& whisper_receiver = whisper["receiver"];
-	for (player_map::const_iterator i = players_.begin(); i != players_.end(); ++i) {
+	for (wesnothd::player_map::const_iterator i = players_.begin(); i != players_.end(); ++i) {
 		if (whisper_receiver != i->second.name().c_str()) {
 			continue;
 		}
 
-		std::vector<game*>::const_iterator g;
+		std::vector<wesnothd::game*>::const_iterator g;
 		for (g = games_.begin(); g != games_.end(); ++g) {
 			if (!(*g)->is_member(i->first)) continue;
 			// Don't send to players in a running game the sender is part of.
@@ -1159,7 +1186,7 @@ void server::process_data_lobby(const network::connection sock,
                                 simple_wml::document& data) {
 	DBG_SERVER << "in process_data_lobby...\n";
 
-	const player_map::iterator pl = players_.find(sock);
+	const wesnothd::player_map::iterator pl = players_.find(sock);
 	if (pl == players_.end()) {
 		ERR_SERVER << "ERROR: Could not find player in players_. (socket: "
 			<< sock << ")\n";
@@ -1180,8 +1207,8 @@ void server::process_data_lobby(const network::connection sock,
 			<< "\tcreates a new game: \"" << game_name << "\".\n";
 		// Create the new game, remove the player from the lobby
 		// and set the player as the host/owner.
-		games_.push_back(new game(players_, sock, game_name));
-		game& g = *games_.back();
+		games_.push_back(new wesnothd::game(players_, sock, game_name));
+		wesnothd::game& g = *games_.back();
 		if(game_password.empty() == false) {
 			g.set_password(game_password);
 		}
@@ -1202,8 +1229,8 @@ void server::process_data_lobby(const network::connection sock,
 		const std::string& password = (*data.root().child("join"))["password"].to_string();
 		int game_id = (*data.root().child("join"))["id"].to_int();
 
-		const std::vector<game*>::iterator g =
-			std::find_if(games_.begin(),games_.end(), game_id_matches(game_id));
+		const std::vector<wesnothd::game*>::iterator g =
+			std::find_if(games_.begin(),games_.end(), wesnothd::game_id_matches(game_id));
 
 		static simple_wml::document leave_game_doc("[leave_game]\n[/leave_game]\n", simple_wml::INIT_COMPRESSED);
 		if (g == games_.end()) {
@@ -1283,14 +1310,14 @@ void server::process_data_game(const network::connection sock,
                                simple_wml::document& data) {
 	DBG_SERVER << "in process_data_game...\n";
 	
-	const player_map::iterator pl = players_.find(sock);
+	const wesnothd::player_map::iterator pl = players_.find(sock);
 	if (pl == players_.end()) {
 		ERR_SERVER << "ERROR: Could not find player in players_. (socket: "
 			<< sock << ")\n";
 		return;
 	}
 
-	std::vector<game*>::iterator itor;
+	std::vector<wesnothd::game*>::iterator itor;
 	for (itor = games_.begin(); itor != games_.end(); ++itor) {
 		if ((*itor)->is_owner(sock) || (*itor)->is_member(sock))
 			break;
@@ -1301,7 +1328,7 @@ void server::process_data_game(const network::connection sock,
 		return;
 	}
 
-	game* g = *itor;
+	wesnothd::game* g = *itor;
 
 	// If this is data describing the level for a game.
 	if (data.root().child("side")) {
@@ -1626,7 +1653,7 @@ void server::process_data_game(const network::connection sock,
 		<< ". (socket:" << sock << ")\n" << data.output();
 }
 
-void server::delete_game(std::vector<game*>::iterator game_it) {
+void server::delete_game(std::vector<wesnothd::game*>::iterator game_it) {
 	metrics_.game_terminated((*game_it)->termination_reason());
 
 	simple_wml::node* const gamelist = games_and_users_list_.child("gamelist");
@@ -1652,12 +1679,12 @@ void server::delete_game(std::vector<game*>::iterator game_it) {
 		LOG_SERVER << "Could not find game (" << (*game_it)->id()
 			<< ") to delete in games_and_users_list_.\n";
 	}
-	const user_vector& users = (*game_it)->all_game_users();
+	const wesnothd::user_vector& users = (*game_it)->all_game_users();
 	// Set the availability status for all quitting users.
-	for (user_vector::const_iterator user = users.begin();
+	for (wesnothd::user_vector::const_iterator user = users.begin();
 		user != users.end(); user++)
 	{
-		const player_map::iterator pl = players_.find(*user);
+		const wesnothd::player_map::iterator pl = players_.find(*user);
 		if (pl != players_.end()) {
 			pl->second.mark_available();
 			if (make_change_diff(games_and_users_list_.root(), NULL,
@@ -1685,7 +1712,7 @@ void server::delete_game(std::vector<game*>::iterator game_it) {
 	games_.erase(game_it);
 }
 
-void server::update_game_in_lobby(const game* g, network::connection exclude)
+void server::update_game_in_lobby(const wesnothd::game* g, network::connection exclude)
 {
 	simple_wml::document diff;
 	if(make_change_diff(*games_and_users_list_.root().child("gamelist"), "gamelist", "game", g->description(), diff)) {
@@ -1711,6 +1738,9 @@ int main(int argc, char** argv) {
 #endif
 	std::string fifo_path = std::string(FIFODIR) + "/socket";
 
+	// setting path to currentworking directory
+	game_config::path = get_cwd();
+
 	// show 'info' by default
 	lg::set_log_domain_severity("server", 2);
 	lg::timestamps(true);
