@@ -1,4 +1,4 @@
-/* $Id: team.cpp 24034 2008-02-24 22:58:45Z jhinrichs $ */
+/* $Id: team.cpp 29170 2008-09-01 17:41:59Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -78,7 +78,7 @@ team::team_info::team_info(const config& cfg) :
 		income(cfg["income"]),
 		income_per_village(),
 		can_recruit(),
-		global_recruitment_pattern(),
+		global_recruitment_pattern(utils::split(cfg["global_recruitment_pattern"])),
 		recruitment_pattern(utils::split(cfg["recruitment_pattern"])),
 		enemies(),
 		team_name(cfg["team_name"]),
@@ -100,6 +100,8 @@ team::team_info::team_info(const config& cfg) :
 		villages_per_scout(),
 		leader_value(0.0),
 		village_value(0.0),
+		aggression_(0.5), 
+		caution_(0.25),
 		targets(),
 		share_maps(false),
 		share_view(false),
@@ -111,10 +113,11 @@ team::team_info::team_info(const config& cfg) :
 {
 	config global_ai_params;
 	const config::child_list& ai_parameters = cfg.get_children("ai");
-	for(config::child_list::const_iterator aiparam = ai_parameters.begin(); aiparam != ai_parameters.end(); ++aiparam) {
+	for(config::child_list::const_reverse_iterator aiparam = ai_parameters.rbegin(); aiparam != ai_parameters.rend(); ++aiparam) {
 		ai_params.push_back(**aiparam);
 
 		if((**aiparam)["turns"].empty() && (**aiparam)["time_of_day"].empty()) {
+//			LOG_NG << "Global ai entry: " << **aiparam << "\n";
 			global_ai_params.append(**aiparam);
 		}
 	}
@@ -190,7 +193,8 @@ team::team_info::team_info(const config& cfg) :
 
 	std::string scouts_val = cfg["villages_per_scout"];
 
-	if(scouts_val.empty()) {
+	if(scouts_val.empty()
+) {
 		scouts_val = global_ai_params["villages_per_scout"];
 	}
 
@@ -202,7 +206,8 @@ team::team_info::team_info(const config& cfg) :
 
 	std::string leader_val = cfg["leader_value"];
 
-	if(leader_val.empty()) {
+	if(leader_val.empty()
+) {
 		leader_val = global_ai_params["leader_value"];
 	}
 
@@ -214,7 +219,8 @@ team::team_info::team_info(const config& cfg) :
 
 	std::string village_val = cfg["village_value"];
 
-	if(village_val.empty()) {
+	if(village_val.empty()
+) {
 		village_val = global_ai_params["village_value"];
 	}
 
@@ -224,16 +230,48 @@ team::team_info::team_info(const config& cfg) :
 		village_value = atof(village_val.c_str());
 	}
 
+	std::string aggression_val = cfg["aggression"];
+
+	if(aggression_val.empty()
+) {
+		aggression_val = global_ai_params["aggression"];
+	}
+
+	if(aggression_val.empty()) {
+		aggression_ = 0.5;
+	} else {
+		aggression_ = atof(aggression_val.c_str());
+	}
+
+	std::string caution_val = cfg["caution"];
+
+	if(caution_val.empty()
+) {
+		caution_val = global_ai_params["caution"];
+	}
+
+	if(caution_val.empty()) {
+		caution_ = 1.0;
+	} else {
+		caution_ = atof(caution_val.c_str());
+	}
+
+
 	std::vector<std::string> recruits = utils::split(cfg["recruit"]);
 	for(std::vector<std::string>::const_iterator i = recruits.begin(); i != recruits.end(); ++i) {
 		can_recruit.insert(*i);
 	}
 	
-	if(recruitment_pattern.empty()) {
+	if(recruitment_pattern.empty()
+) {
 		recruitment_pattern = 
 			utils::split(global_ai_params["recruitment_pattern"]);
+		LOG_NG << "Recruitment pattern: " << global_ai_params["recruitment_pattern"] << "\n";
+	} else {
+		LOG_NG << "Recruitment pattern from side: " << cfg["recruitment_pattern"] << "\n";
 	}
 
+
 /*	// Default recruitment pattern is to buy 2 fighters for every 1 archer
 	if(recruitment_pattern.empty()) {
 		recruitment_pattern.push_back("fighter");
@@ -314,6 +352,8 @@ void team::team_info::write(config& cfg) const
 	cfg["villages_per_scout"] = str_cast(villages_per_scout);
 	cfg["leader_value"] = str_cast(leader_value);
 	cfg["village_value"] = str_cast(village_value);
+	cfg["aggression"] = str_cast(aggression_);
+	cfg["caution"] = str_cast(caution_);
 
 	for(std::vector<target>::const_iterator tg = targets.begin(); tg != targets.end(); ++tg) {
 		tg->write(cfg.add_child("target"));
@@ -329,10 +369,21 @@ void team::team_info::write(config& cfg) const
 
 	cfg["recruit"] = can_recruit_str.str();
 
+	std::stringstream global_recruit_pattern_str;
+	for(std::vector<std::string>::const_iterator p = global_recruitment_pattern.begin();
+		   	p != global_recruitment_pattern.end(); ++p) {
+		if(p != global_recruitment_pattern.begin())
+			global_recruit_pattern_str << ",";
+
+		global_recruit_pattern_str << *p;
+	}
+
+	cfg["global_recruitment_pattern"] = global_recruit_pattern_str.str();
+
 	std::stringstream recruit_pattern_str;
-	std::vector<std::string> rp = global_recruitment_pattern;
-	for(std::vector<std::string>::const_iterator p = rp.begin(); p != rp.end(); ++p) {
-		if(p != rp.begin())
+	for(std::vector<std::string>::const_iterator p = recruitment_pattern.begin();
+		   	p != recruitment_pattern.end(); ++p) {
+		if(p != recruitment_pattern.begin())
 			recruit_pattern_str << ",";
 
 		recruit_pattern_str << *p;
@@ -359,8 +410,6 @@ team::team(const config& cfg, int gold) :
 		countdown_time_(0),
 		action_bonus_count_(0),
 		aiparams_(),
-		aggression_(0.0), 
-		caution_(0.0),
 		enemies_(),
 		seen_(),
 		ally_shroud_(),
@@ -461,8 +510,8 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 	info_.recruitment_pattern = utils::split(aiparams_["recruitment_pattern"]);
 	if (info_.recruitment_pattern.empty())
 		info_.recruitment_pattern = info_.global_recruitment_pattern;
-	aggression_ = lexical_cast_default<double>(aiparams_["aggression"],0.5);
-	caution_ = lexical_cast_default<double>(aiparams_["caution"],0.25);
+	info_.aggression_ = lexical_cast_default<double>(aiparams_["aggression"],0.5);
+	info_.caution_ = lexical_cast_default<double>(aiparams_["caution"],0.25);
 }
 
 bool team::calculate_enemies(size_t index) const
