@@ -1,4 +1,4 @@
-/* $Id: unit.cpp 27191 2008-06-15 09:46:11Z boucman $ */
+/* $Id: unit.cpp 29165 2008-09-01 12:53:23Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -2784,6 +2784,18 @@ void unit::apply_modifications()
 	}
 }
 
+void unit::ambush(const gamemap::location& loc) const
+{
+	invisibility_cache_.find(loc);
+	std::vector<const unit *>::iterator itor =
+	std::find(units_with_cache.begin(), units_with_cache.end(), this);
+
+	if(itor != units_with_cache.end()) {
+		units_with_cache.erase(itor);
+	}
+
+}
+
 bool unit::invisible(const gamemap::location& loc,
 		const unit_map& units,const std::vector<team>& teams, bool see_all) const
 {
