@@ -1,4 +1,4 @@
-/* $Id: unit.hpp 24177 2008-03-01 15:45:18Z martinxyz $ */
+/* $Id: unit.hpp 29165 2008-09-01 12:53:23Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -269,6 +269,11 @@ public:
 	// Only see_all=true use caching
 	bool invisible(const gamemap::location& loc,
 		const unit_map& units,const std::vector<team>& teams, bool see_all=true) const;
+	/**
+	 * Make sure that invisibility cache is revalidate
+	 * after ambush;
+	 **/
+	void ambush(const gamemap::location& loc) const;
 
 	unit_race::GENDER generate_gender(const unit_type& type, bool gen, game_state* state = 0);
 	std::string image_mods() const;
