@@ -1,10 +1,11 @@
-/* $Id: button.cpp 15790 2007-02-14 23:55:53Z jhinrichs $ */
+/* $Id: button.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,15 +15,19 @@
 #include "global.hpp"
 
 #include "widgets/button.hpp"
+#include "game_config.hpp"
 #include "font.hpp"
 #include "marked-up_text.hpp"
 #include "image.hpp"
 #include "log.hpp"
+#include "sound.hpp"
 #include "util.hpp"
 #include "video.hpp"
 #include "wml_separators.hpp"
 #include "serialization/string_utils.hpp"
 
+#define ERR_DP LOG_STREAM(err, display)
+
 namespace gui {
 
 const int font_size = font::SIZE_SMALL;
@@ -31,10 +36,10 @@ const int checkbox_horizontal_padding = font::SIZE_SMALL / 2;
 const int vertical_padding = font::SIZE_SMALL / 2;
 
 button::button(CVideo& video, const std::string& label, button::TYPE type,
-               std::string button_image_name, SPACE_CONSUMPTION spacing)
-	: widget(video), label_(label),
+               std::string button_image_name, SPACE_CONSUMPTION spacing, const bool auto_join)
+	: widget(video, auto_join), type_(type), label_(label),
 	  image_(NULL), pressedImage_(NULL), activeImage_(NULL), pressedActiveImage_(NULL),
-	  button_(true), state_(NORMAL), type_(type), pressed_(false),
+	  button_(true), state_(NORMAL), pressed_(false),
 	  spacing_(spacing), base_height_(0), base_width_(0)
 {
 	if(button_image_name.empty() && type == TYPE_PRESS) {
@@ -44,9 +49,9 @@ button::button(CVideo& video, const std::string& label, button::TYPE type,
 	}
 
 	const std::string button_image_file = "buttons/" + button_image_name + ".png";
-	surface button_image(image::get_image(button_image_file,image::UNSCALED));
-	surface pressed_image(image::get_image("buttons/" + button_image_name + "-pressed.png", image::UNSCALED));
-	surface active_image(image::get_image("buttons/" + button_image_name + "-active.png", image::UNSCALED));
+	surface button_image(image::get_image(button_image_file));
+	surface pressed_image(image::get_image("buttons/" + button_image_name + "-pressed.png"));
+	surface active_image(image::get_image("buttons/" + button_image_name + "-active.png"));
 	surface pressed_active_image;
 
 	if (pressed_image.null())
@@ -56,14 +61,15 @@ button::button(CVideo& video, const std::string& label, button::TYPE type,
 		active_image.assign(button_image);
 
 	if (type == TYPE_CHECK) {
-		pressed_active_image.assign(image::get_image("buttons/" + button_image_name + "-active-pressed.png",
-		                                             image::UNSCALED));
+		pressed_active_image.assign(image::get_image("buttons/" + button_image_name + "-active-pressed.png"));
 		if (pressed_active_image.null())
 			pressed_active_image.assign(pressed_image);
 	}
 
-	if (button_image.null())
+	if (button_image.null()) {
+		ERR_DP << "error initializing button!\n";
 		throw error();
+	}
 
 	base_height_ = button_image->h;
 	base_width_ = button_image->w;
@@ -104,7 +110,7 @@ void button::calculate_size()
 	if (!change_size) {
 		unsigned w = loc.w - (type_ == TYPE_PRESS ? horizontal_padding : checkbox_horizontal_padding + base_width_);
 		if (type_ != TYPE_IMAGE){
-			label_ = font::make_text_ellipsis(label_, font_size, w);
+			label_ = font::make_text_ellipsis(label_, font_size, w, false);
 		}
 	}
 
@@ -132,7 +138,11 @@ void button::calculate_size()
 		}
 #endif
 	} else {
-		set_width(checkbox_horizontal_padding + textRect_.w + base_width_);
+		if(label_.empty()) {
+			set_width(base_width_);
+		} else {
+			set_width(checkbox_horizontal_padding + textRect_.w + base_width_);
+		}
 	}
 }
 
@@ -157,7 +167,10 @@ void button::enable(bool new_val)
 	if(new_val != enabled())
 	{
 		pressed_ = false;
-		state_ = NORMAL;
+		// check buttons should keep their state
+		if(type_ != TYPE_CHECK) {
+			state_ = NORMAL;
+		}
 		widget::enable(new_val);
 	}
 }
@@ -199,8 +212,10 @@ void button::draw_contents()
 	SDL_Color button_colour = font::BUTTON_COLOUR;
 
 	if (!enabled()) {
-		image = greyscale_image(image);
-		button_colour = font::STONED_COLOUR;
+		static const Uint32 disabled_btn_color = 0xAAAAAA;
+		static const double disabled_btn_adjust = 0.18;
+		image = blend_surface(greyscale_image(image), disabled_btn_adjust, disabled_btn_color);
+		button_colour = font::GRAY_COLOUR;
 	}
 
 	video().blit_surface(loc.x, loc.y, image);
@@ -220,9 +235,7 @@ bool button::hit(int x, int y) const
 	return point_in_rect(x,y,location());
 }
 
-namespace {
-	bool not_image(const std::string& str) { return !str.empty() && str[0] != IMAGE_PREFIX; }
-}
+static bool not_image(const std::string& str) { return !str.empty() && str[0] != IMAGE_PREFIX; }
 
 void button::set_label(const std::string& val)
 {
@@ -263,6 +276,7 @@ void button::mouse_down(SDL_MouseButtonEvent const &event)
 {
 	if (hit(event.x, event.y) && event.button == SDL_BUTTON_LEFT && type_ != TYPE_CHECK){
 		state_ = PRESSED;
+		sound::play_UI_sound(game_config::sounds::button_press);
 	}
 }
 
@@ -280,6 +294,7 @@ void button::mouse_up(SDL_MouseButtonEvent const &event)
 	case TYPE_CHECK:
 		state_ = state_ == ACTIVE ? PRESSED_ACTIVE : ACTIVE;
 		pressed_ = true;
+		sound::play_UI_sound(game_config::sounds::checkbox_release);
 		break;
 	case TYPE_PRESS:
 		if (state_ == PRESSED) {
